
//////////////////////////////////////////////////////////////////////////////
// Includes

#include "fluids/matrixBuilder.h"
#include "util/assert.h"


//////////////////////////////////////////////////////////////////////////////

MatrixMultiplier::MatrixMultiplier( size_t r, size_t c )
{
    numRows = r;
    numColumns = c;
    rows.resize( numRows );
}

void MatrixMultiplier::Set( size_t i, size_t j, real val )
{
    rows[i][j] = val;
}

void MatrixMultiplier::Add( size_t i, size_t j, real val )
{
    rows[i][j] += val;
}

void DiagonalMatrix::Invert()
{
    for( size_t i = 0; i < size(); ++i )
        (*this)[i] = (real)1.0 / (*this)[i];
}

MatrixMultiplier operator*( const MatrixMultiplier& src1, const MatrixMultiplier& src2 )
{
    MatrixMultiplier result( src1.numRows, src2.numColumns );
    for( size_t i = 0; i < src1.numRows; ++i )
        for( std::map<size_t,real>::const_iterator t1 = src1.rows[i].begin(); t1 != src1.rows[i].end(); ++t1 )
            for( std::map<size_t,real>::const_iterator t2 = src2.rows[t1->first].begin(); t2 != src2.rows[t1->first].end(); ++t2 )
                result.Add( i, t2->first, t1->second * t2->second );
    return result;
}

MatrixMultiplier operator*( const MatrixMultiplier& src1, const DiagonalMatrix& src2 )
{
    MatrixMultiplier result( src1.numRows, src1.numColumns );
    for( size_t i = 0; i < src1.numRows; ++i )
        for( std::map<size_t,real>::const_iterator t1 = src1.rows[i].begin(); t1 != src1.rows[i].end(); ++t1 )
            result.Set( i, t1->first, t1->second * src2[t1->first] );
    return result;
}

MatrixMultiplier operator*( const DiagonalMatrix& src1, const MatrixMultiplier& src2 )
{
    MatrixMultiplier result( src2.numRows, src2.numColumns );
    for( size_t i = 0; i < src2.numRows; ++i )
        for( std::map<size_t,real>::const_iterator t2 = src2.rows[i].begin(); t2 != src2.rows[i].end(); ++t2 )
            result.Set( i, t2->first, t2->second * src1[i] );
    return result;
}

void MatrixMultiplier::operator+=( const MatrixMultiplier& mat )
{
    for( size_t i = 0; i < mat.numRows; ++i )
        for( std::map<size_t,real>::const_iterator t2 = mat.rows[i].begin(); t2 != mat.rows[i].end(); ++t2 )
            Add( i, t2->first, t2->second );
}
