
//////////////////////////////////////////////////////////////////////////////
// Includes

#include "simplicialComplex/edge.h"
#include "simplicialComplex/simplicialComplex.h"
#include "simplicialComplex/edgeFaceIterator.h"
#include "simplicialComplex/edgeTetIterator.h"
#include "simplicialComplex/vertex.h"
#include "util/assert.h"


//////////////////////////////////////////////////////////////////////////////
// Default Constructor

Edge::Edge() : sc(0), index(size_t(-1))
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// Private Constructor that takes a mesh pointer and index

Edge::Edge( const SimplicialComplex<3> *sc, size_t index ) : sc(sc), index(index)
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

Edge::~Edge()
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// operator bool - Is this face valid

Edge::operator bool() const
{
    return (sc && index < sc->NumEdges());
}

//////////////////////////////////////////////////////////////////////////////
// Center

vec3 Edge::Center() const
{
    smart_assert(*this).msg( "Invalid edge" ).fatal();
    return sc->m_edges[index].Center();
}

//////////////////////////////////////////////////////////////////////////////
// Length

real Edge::Length() const
{
    smart_assert(*this).msg( "Invalid edge" ).fatal();
    return sc->m_edges[index].Volume();
}

//////////////////////////////////////////////////////////////////////////////
// DualArea

real Edge::DualArea() const
{
    smart_assert(*this).msg( "Invalid edge" ).fatal();
    return sc->m_edges[index].DualVolume();
}

//////////////////////////////////////////////////////////////////////////////
// Vertices

void Edge::Vertices( vec3& v0, vec3& v1 ) const
{
    smart_assert(*this).msg( "Invalid edge" ).fatal();
    v0 = sc->m_vertices[ sc->m_edges[index].Get(0) ].Center();
    v1 = sc->m_vertices[ sc->m_edges[index].Get(1) ].Center();
}

//////////////////////////////////////////////////////////////////////////////
// Vertices

void Edge::Vertices( Vertex& v0, Vertex& v1 ) const
{
    smart_assert(*this).msg( "Invalid edge" ).fatal();
    v0.sc = v1.sc = sc;
    v0.index = sc->m_edges[index].Get(0);
    v1.index = sc->m_edges[index].Get(1);
}

//////////////////////////////////////////////////////////////////////////////
// Faces

EdgeFaceIterator Edge::Faces( void ) const
{
    return EdgeFaceIterator(*this);
}

//////////////////////////////////////////////////////////////////////////////
// Tets

EdgeTetIterator Edge::Tets( void ) const
{
    return EdgeTetIterator(*this);
}

//////////////////////////////////////////////////////////////////////////////
// IsOnBoundary

bool Edge::IsOnBoundary() const
{
    smart_assert(*this).msg( "Invalid edge" ).fatal();
    // It's a boundary edge if its tet list contains an invalid tet
    return ( sc->m_ETmatrix[ index ].back() == size_t(-1) );
}

//////////////////////////////////////////////////////////////////////////////
// Equality operator

bool operator==( const Edge& lhs, const Edge& rhs )
{
    return (lhs.sc == rhs.sc && lhs.index == rhs.index);
}

//////////////////////////////////////////////////////////////////////////////
// Inequality operator

bool operator!=( const Edge& lhs, const Edge& rhs )
{
    return (!(lhs == rhs));
}
