#ifndef _SIMPLICIAL_COMPLEX_EDGE_FACE_ITERATOR_H_
#define _SIMPLICIAL_COMPLEX_EDGE_FACE_ITERATOR_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "simplicialComplex/face.h"
#include <vector>

class Edge;


//////////////////////////////////////////////////////////////////////////////
// class EdgeFaceIterator
//

class EdgeFaceIterator
{
public:  // methods
    EdgeFaceIterator( const Edge& v );
    ~EdgeFaceIterator() {}

    EdgeFaceIterator& operator++();
    EdgeFaceIterator operator++(int);

    const Face& operator*() const;
    const Face* operator->() const;

    int RelativeSign() const;

    operator bool() const;

private:  // methods
    EdgeFaceIterator();

private:  // data
    std::vector< std::pair<size_t,int> >::const_iterator m_iter;
    std::vector< std::pair<size_t,int> >::const_iterator m_end;
    Face m_face;
};


inline EdgeFaceIterator& EdgeFaceIterator::operator++()
{
    ++m_iter;
    if(*this) m_face.index = m_iter->first;
    else      m_face.index = size_t(-1);
    return (*this);
}

inline EdgeFaceIterator EdgeFaceIterator::operator++(int)
{
    EdgeFaceIterator iterator = *this;
    ++(*this);
    return iterator;
}

inline const Face& EdgeFaceIterator::operator*() const
{
    return m_face;
}

inline const Face* EdgeFaceIterator::operator->() const
{
    return &m_face;
}

inline int EdgeFaceIterator::RelativeSign() const
{
    return m_iter->second;
}

inline EdgeFaceIterator::operator bool() const
{
    return (m_iter != m_end);
}


#endif  // _SIMPLICIAL_COMPLEX_EDGE_FACE_ITERATOR_H_
