#include "simplicialComplex/io/tetFile.h"
#include "simplicialComplex/simplicialComplex.h"
#include "simplicialComplex/meshFactory.h"
#include "util/linear.h"
#include "util/tokenizer.h"
#include "util/default_logs.h"

#include <vector>
#include <sstream>

using std::vector;


static bool ReadHeader( Tokenizer& token, int& numVertices, int& numTets )
{
    // first line: "v vertices"
    if( !token.next().value(&numVertices) )
        return false;
    if( token.next() != "vertices" )
        return false;

    // second line: "t tets"
    if( !token.next().value(&numTets) )
        return false;
    if( token.next() != "tets" )
        return false;

    return true;
}

static bool ReadGeometry( Tokenizer& token, vector<vec3>& vertices )
{
    for( size_t i = 0; i < vertices.size(); ++i )
    {
        if( !token.next().value( &vertices[i][0] ) ||
            !token.next().value( &vertices[i][1] ) ||
            !token.next().value( &vertices[i][2] ) )
            return false;
    }

    return true;
}

static bool ReadTopology( Tokenizer& token, vector< vec<4,int> >& tets )
{
    int temp;
    for( size_t i = 0; i < tets.size(); ++i )
    {
        if( !token.next().value( &temp ) || temp != 4 )
            return false;

        if( !token.next().value( &tets[i][0] ) ||
            !token.next().value( &tets[i][1] ) ||
            !token.next().value( &tets[i][2] ) ||
            !token.next().value( &tets[i][3] ) )
            return false;
    }

    return true;
}


bool TET_File::Read( SimplicialComplex<3>& mesh, std::istream& in )
{
    int numVertices, numTets;
    vector< vec3 > vertices;
    vector< vec<4,int> > tets;

    Tokenizer token(in);

    if( !ReadHeader(token, numVertices, numTets) )
    {
        LOG(err) << "Error reading line " << token.line() << " of TET file";
        return false;
    }

    vertices.resize( numVertices );
    tets.resize( numTets );

    if( !ReadGeometry(token, vertices) || !ReadTopology(token, tets) )
    {
        LOG(err) << "Error reading line " << token.line() << " of TET file";
        return false;
    }

    LOG(info) << "Building connectivity";
    MeshFactory<3> factory( mesh );
    if( !factory.Build(vertices, tets) )
    {
        LOG(err) << "Bad Mesh";
        return false;
    }

    return true;
}
