#ifndef _SIMPLICIAL_COMPLEX_H_
#define _SIMPLICIAL_COMPLEX_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "simplicialComplex/simplex.h"
#include "util/linear.h"

#include <vector>


//////////////////////////////////////////////////////////////////////////////

template<typename T>
class SimplexIterator;


//////////////////////////////////////////////////////////////////////////////
// class SimplicialComplex
// Template class for an N-dimensional simplicial complex.

template<int N>
class SimplicialComplex
{
};

template<>
class SimplicialComplex<3>
{
    // friends
    template<int N> friend class MeshFactory;
    friend class Vertex;
    friend class Edge;
    friend class Face;
    friend class Tet;
    friend class VertexEdgeIterator;
    friend class EdgeFaceIterator;
    friend class EdgeTetIterator;
    friend class MatrixBuilder;

public:  // methods
    SimplicialComplex();
    ~SimplicialComplex();

    size_t NumVerts( void ) const  { return m_vertices.size(); }
    size_t NumEdges( void ) const  { return m_edges.size(); }
    size_t NumFaces( void ) const  { return m_faces.size(); }
    size_t NumTets ( void ) const  { return m_tets.size(); }

    SimplexIterator<Vertex> Vertices( void ) const;
    SimplexIterator<Edge>   Edges   ( void ) const;
    SimplexIterator<Face>   Faces   ( void ) const;
    SimplexIterator<Tet>    Tets    ( void ) const;

    Vertex GetVertex( size_t index ) const;
    Edge   GetEdge  ( size_t index ) const;
    Face   GetFace  ( size_t index ) const;
    Tet    GetTet   ( size_t index ) const;

private:  // typedefs
    typedef std::vector< Simplex<0> > S0List;
    typedef std::vector< Simplex<1> > S1List;
    typedef std::vector< Simplex<2> > S2List;
    typedef std::vector< Simplex<3> > S3List;

    typedef std::pair<size_t,int> SignedIndex;

private:  // data

    // All the simplices in the complex.
    // Simplices are represented by integer tuples that refer to
    // indices in the vector of 0-simplices.
    S0List m_vertices;
    S1List m_edges;
    S2List m_faces;
    S3List m_tets;

    // The boundary operator cache. These maps store the neighbor
    // relationships between k- and (k-1)-simplices in a sparse
    // matrix representation.
    std::vector< vec<2,size_t> >      m_EVmatrix;  // the negative one is first, positive second
    std::vector< vec<3,SignedIndex> > m_FEmatrix;
    std::vector< vec<4,SignedIndex> > m_TFmatrix;

    // The coboundary operators. These matrices are just the transposes
    // of the three above. I'm not sure we need all of these, but
    // they're in for now for the sake of completeness.
    std::vector< std::vector<SignedIndex> > m_VEmatrix;
    std::vector< std::vector<SignedIndex> > m_EFmatrix;
    std::vector< vec<2,size_t> >            m_FTmatrix;  // the negative one is first, positive second

    // This is the composition of EF and FT.
    // Unlike the others, this one is unsigned. The notion of relative
    // orientation of a tet to an edge is not well-defined.
    std::vector< std::vector<size_t> > m_ETmatrix;
};


#endif  // _SIMPLICIAL_COMPLEX_H_
