#ifndef _SIMPLICIAL_COMPLEX_TET_H_
#define _SIMPLICIAL_COMPLEX_TET_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "util/linear.h"


//////////////////////////////////////////////////////////////////////////////

template<int N>
class SimplicialComplex;
class Vertex;


//////////////////////////////////////////////////////////////////////////////
// class Tet
// A class for doing operations on and querying a 3-simplex of a complex

class Tet
{
    // friends
    template<typename T>
    friend class SimplexIterator;
    friend class EdgeTetIterator;
    friend class SimplicialComplex;
    friend class Face;

    friend bool operator==( const Tet&, const Tet& );
    friend bool operator!=( const Tet&, const Tet& );

public:  // methods
    Tet();
    ~Tet();

    operator bool() const;

    size_t Index( void ) const  { return index; }

    vec3 Center( void ) const;
    real Volume( void ) const;

    void Vertices( vec3& v0, vec3& v1, vec3& v2, vec3& v3 ) const;
    void Vertices( int& v0, int& v1, int& v2, int& v3 ) const;
    void Vertices( Vertex& v0, Vertex& v1, Vertex& v2, Vertex& v3 ) const;
    void Faces( Face& f0, Face& f1, Face& f2, Face& f3 ) const;

    Vertex OppositeVertex( const Face& face ) const;
    Tet AdjacentTet( const Face& face ) const;
    Tet AdjacentTet( int whichNeighbor ) const;

    bool IsOnBoundary( void ) const;

private:  // methods
    Tet( const SimplicialComplex<3> *sc, size_t index );

private:  // data
    const SimplicialComplex<3> *sc;
    size_t index;
};


#endif  // _SIMPLICIAL_COMPLEX_TET_H_
