#ifndef _SIMPLICIAL_COMPLEX_VERTEX_EDGE_ITERATOR_H_
#define _SIMPLICIAL_COMPLEX_VERTEX_EDGE_ITERATOR_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "simplicialComplex/edge.h"
#include <vector>

class Vertex;


//////////////////////////////////////////////////////////////////////////////
// class VertexEdgeIterator
//

class VertexEdgeIterator
{
public:  // methods
    VertexEdgeIterator( const Vertex& v );
    ~VertexEdgeIterator() {}

    VertexEdgeIterator& operator++();
    VertexEdgeIterator operator++(int);

    const Edge& operator*() const;
    const Edge* operator->() const;

    int RelativeSign() const;

    operator bool() const;

private:  // methods
    VertexEdgeIterator();

private:  // data
    std::vector< std::pair<size_t,int> >::const_iterator m_iter;
    std::vector< std::pair<size_t,int> >::const_iterator m_end;
    Edge m_edge;
};


inline VertexEdgeIterator& VertexEdgeIterator::operator++()
{
    ++m_iter;
    if(*this) m_edge.index = m_iter->first;
    else      m_edge.index = size_t(-1);
    return (*this);
}

inline VertexEdgeIterator VertexEdgeIterator::operator++(int)
{
    VertexEdgeIterator iterator = *this;
    ++(*this);
    return iterator;
}

inline const Edge& VertexEdgeIterator::operator*() const
{
    return m_edge;
}

inline const Edge* VertexEdgeIterator::operator->() const
{
    return &m_edge;
}

inline int VertexEdgeIterator::RelativeSign() const
{
    return m_iter->second;
}

inline VertexEdgeIterator::operator bool() const
{
    return (m_iter != m_end);
}


#endif  // _SIMPLICIAL_COMPLEX_VERTEX_EDGE_ITERATOR_H_
